<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class ParaTipoConsumidor extends Crud{
    
    protected $dataSourceName  = '';
    const  TIPO_GERAL = 'paraTipoConsumidorGeral';
    const  TIPO_CONSUMIDOR = 'paraTipoConsumidor';
    const TIPO_POR_FILIAL = 'paraTipoConsumidorPorFilial';
    const TAM_MINIMO_STRING = 9;
    
    public function __construct (Manager $dataSourceManager, Utilities\Util $utilService, Services\ParaTipoConsumidorService $paraTipoConsumidorService){
        parent::__construct($dataSourceManager);
        $this->paraConsumidor = $paraTipoConsumidorService;
        $this->utilService = $utilService;
    }
    
    public function findConsumidor(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_CONSUMIDOR;
        parent::find($request,$response);
    }
    
    public function saveConsumidor(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_CONSUMIDOR;
        $this->save($request,$response);
    }
    
    public function deleteConsumidor(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_CONSUMIDOR;
        parent::delete($request,$response);
    }
    
    public function findGeral(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_GERAL;
        parent::find($request,$response);
    }
    
    public function saveGeral(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_GERAL;
        $this->save($request,$response);
    }
    
    public function deleteGeral(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_GERAL;
        parent::delete($request,$response);
    }
    
    public function findPorFilial(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = self::TIPO_POR_FILIAL;
        parent:: find($request,$response);
        $dataset = current($response->getDataSets());
        $newArray =array();
        forEach($dataset->getRows() as $k => $value){
            $newArray[$k] = $value;
            for($i = 0; $i < 9; $i++){
                $newArray[$k]['POS_'.$i] = $value['DSACESSOSITE'][$i];
                $newArray[$k]['SEL_'.$i] = (int)$value['DSPOSMNUSITE'][$i];
            }
            
        }
        $response->addDataSet(new DataSet($this->dataSourceName, $newArray));
    }
    
    public function savePorFilial(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        if(isset($rows[0]['INSERT'])){
            $params = array(
                ':CDFILIAL' => $rows[0]['CDFILIAL'],
                ':CDTIPOCONS' => $rows[0]['CDTIPOCONS'],
                ':DSACESSOSITE' => 'NNNNNNNNNSSSSSSSSSSS',
                ':DSPOSMNUSITE' => '123456789',
                ':CDTIPOCONSEX' => $rows[0]['CDTIPOCONSEX'],
                ':IDRECEBEBONUS' => 'N',
                ':VRCARGACREDITO' => null,
                ':VRBONUSCREDITO' => null,
                ':VRSALDCONSBONUS' => null,
                ':IDPERIODOBONUS' => 'M',
                ':CDTIPORECE' => null,
                ':QTRECARGASMES' => null);
            $result = $this->paraConsumidor->savePorFilial($params);
            //$response->addDataSet(new DataSet('NewConsumidor', $result));
        }
        else{
            $this->dataSourceName = self::TIPO_POR_FILIAL;
            if(strlen($rows[0]['DSACESSOSITE']) >= self::TAM_MINIMO_STRING && $this->validarSequenciaNumerica($rows[0]['DSPOSMNUSITE'])){
                $this->save($request,$response);
            }else{
                $response->setError(new DTO\Response\Error("Não foi possivel atualizar", 1,null));
            }
        }
    }
    
    private function validarSequenciaNumerica ($numbers){
        if(strlen($numbers) < self::TAM_MINIMO_STRING) return false;
        
        $numbers = str_split($numbers);
        $array = array();
        foreach ($numbers as $value) {
            if(in_array($value,$array)) return false;
            $array[] = $value;
        }
        return true;
    }
    
    public function findNewConsumidorFilial(DTO\Request $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $NRORG = $conditions['NRORG'];
        $CDFILIAL = $conditions['CDFILIAL'];
        $params = array(
            ':CDFILIAL' => $CDFILIAL,
            ':NRORG' => $NRORG);
        $result = $this->paraConsumidor->findNewConsumidorFilial($params);
        $response->addDataSet(new DataSet('Select_Cartoes',$result));
    }
    
    public function removePorFilial(DTO\Request $request, DTO\Response $response){
        $conditions = $request->getRow();
        $params = array(
            ':CDFILIAL' => $conditions['CDFILIAL'],
            ':CDTIPOCONS' => $conditions['CDTIPOCONS'],
            ':NRORG' => $conditions['NRORG']);
        $result = $this->paraConsumidor->removePorFilial($params);
    }
    
    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}