<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;


class RelVendasProdutoPorConsumidor{
    
    const PARAM_P_TELA = 'P_';
    const PARAM_PT_TELA = 'P_T';
    protected $relatorioService;
    protected $relMapaService;
    protected $util;
    protected $birtReport;
    protected $nameReport = 'VendaDeProdutosPorConsumidor';
    protected $retirarParams = array('NMLOJA','NMFILIAL','changed','PERIODO','CDFILIAL','CDCAIXA','CDLOJA',
    'NMRAZSOCCLIE','NRINSJURCLIE','CDCLIENTE','FILILOJA','CDCONSUMIDOR','NMCONSUMIDOR','CLIENTECONSUMIDOR','CDPRODUTO','NMPRODUTO',
    'CDTIPOCONS','NMTIPOCONSCLI');
    protected $checkParams = array('CDPRODUTO','CDCLIENTE','CDCONSUMIDOR','CDTIPOCONS','CDLOJA','FILILOJA');
    protected $inserirParams = array(
        'NOMEINI'=> null,
        'NOMEFIM' => null
    );
    
    public function __construct (Utilities\Util $util,ReportService $reportService, Services\Relatorio $relatorioService, 
    BirtReport $birtReport) {
        $this->reportService = $reportService;
        $this->relatorioService = $relatorioService;
        $this->util = $util;
        $this->birtReport = $birtReport;
    }
    
    public function gerarRelatorio(DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        $conditions['P_CDFILIAL'] = $conditions['CDFILIAL'];
        $conditions['CDPICTPROD'] = str_replace(array('*',9),'@',$conditions['CDPICTPROD']);
        $conditions = $this->buildSubstutirParams($conditions);
        // $conditions = $this->buildParams($conditions);
        $conditions = $this->buildPeriodo($conditions);
        // $conditions = $this->retirarParams($conditions);
        $conditions = $this->inserirParams($conditions);
        $conditions['__report_name'] = $this->getNameReport(isset($conditions['TIPORELATORIO'])?$conditions['TIPORELATORIO']:"Analitico");
        $params = $this->createParamsArray($conditions);
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $conditions['__report_name']
        );
        $this->getRelatorio($params,$response);
    }
    
    private function buildSubstutirParams($conditions){
        if(isset($conditions['CDLOJA']) && (\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDLOJA', $conditions) || count($conditions['CDLOJA']))) {
            $conditions['CDLOJA'] = $conditions['FILILOJA'];
        }
        if(isset($conditions['CDCONSUMIDOR']) && (\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDCONSUMIDOR', $conditions) || count($conditions['CDCONSUMIDOR']))) {
            $conditions['CDCONSUMIDOR'] = $conditions['CLIENTECONSUMIDOR'];
        }
        return $conditions;
    }
    
    private function parameterMapping($conditions){
        $parameterMapping = array();
        foreach ($conditions as $key=>$value) {
            $parameterMapping[$key] = $key;
        }
        unset($parameterMapping['__report_name']);
        return $parameterMapping;
    }
    
    public function getRelatorio ($relatorioData , DTO\Response $response){
         try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $relatorioData['ReportName'], $relatorioData['ParametersRow']);
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
     }
     
    private function insereP ($text){
        return self::PARAM_P_TELA.$text;
    }
    
    private function insereP_T ($text){
        return self::PARAM_PT_TELA.$text;
    }
    
    private function getNameReport($name){
        if($name==''){
            return $this->nameReport;
        }
        return $this->nameReport.$name;
    }
    
    private function buildParams ($conditions){
        foreach ($this->checkParams as $value) {
            if(!isset($value, $conditions) || $conditions[$value]=!0) {
                $conditions[$this->insereP($value)] = null;
                $conditions[$this->insereP_T($value)] = 'T';
            }else{
                $conditions[$this->insereP($value)] = implode('_', $conditions[$value]);
                $conditions[$this->insereP_T($value)] = null;
            }
        }
        return $conditions;
    }
     
    private function buildPeriodo ($conditions){
        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('PERIODO', $conditions)) {
            $conditions[$this->insereP('DTLANCINI')] = $conditions['PERIODO']['start'];
            $conditions[$this->insereP('DTLANCFIN')] = $conditions['PERIODO']['end'];
        }
        return $conditions;
    }
    
    private function retirarParams($conditions){
        foreach ($this->retirarParams as $value) {
            if(isset($conditions[$value])){
                unset($conditions[$value]);
            }
        }
        return $conditions;
    }
    
    private function inserirParams($conditions){
        foreach ($this->inserirParams as $key =>  $value) {
            $conditions[$this->insereP($key)] = $value;
        }
        return $conditions;
    }
    
    private function createParamsArray($conditions){
        $params = array (
            "NRORG"           => $conditions["NRORG"],
            "CDPICTPROD"      => $conditions["CDPICTPROD"],
            "P_CDFILIAL"      => $conditions["P_CDFILIAL"],
            "P_CDPRODUTO"     => !empty($conditions["CDPRODUTO"]) ? implode('_', $conditions["CDPRODUTO"]) : '',
            "P_TCDPRODUTO"    => !empty($conditions["CDPRODUTO"]) ? '' : 'T',
            "P_CDCLIENTE"     => !empty($conditions["CDCLIENTE"]) ? implode('_', $conditions["CDCLIENTE"]) : '',
            "P_TCDCLIENTE"    => !empty($conditions["CDCLIENTE"]) ? '' : 'T',
            "P_CDCONSUMIDOR"  => !empty($conditions["CDCONSUMIDOR"]) ? implode('_', $conditions["CDCONSUMIDOR"]) : '',
            "P_TCDCONSUMIDOR" => !empty($conditions["CDCONSUMIDOR"]) ? '' : 'T',
            "P_CDTIPOCONS"    => !empty($conditions["CDTIPOCONS"]) ? implode('_', $conditions["CDTIPOCONS"]) : '',
            "P_TCDTIPOCONS"   => !empty($conditions["CDTIPOCONS"]) ? '' : 'T',
            "P_CDLOJA"        => !empty($conditions["CDLOJA"]) ? implode('_', $conditions["CDLOJA"]) : '',
            "P_TCDLOJA"       => !empty($conditions["CDLOJA"]) ? '' : 'T',
            "P_FILILOJA"      => !empty($conditions["FILILOJA"]) ? implode('_', $conditions["FILILOJA"]) : '',
            "P_TFILILOJA"     => !empty($conditions["FILILOJA"]) ? '' : 'T',
            "P_DTLANCINI"     => $conditions["P_DTLANCINI"],
            "P_DTLANCFIN"     => $conditions["P_DTLANCFIN"],
            "P_NOMEINI"       => $conditions["P_NOMEINI"],
            "P_NOMEFIM"       => $conditions["P_NOMEFIM"]
        );
        return $params;
    }
}