<?php

namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;
use Teknisa\MAN\Utilities\DateUtil;
use \Setforms2\Shared\ORM\DateTime;

class FechamentoDiaForSaleService {
	
	protected $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
		$this->entityManager = $entityManager;
	}
	
	public function getVigenciaInterval( $NRORG, $CDTABEPREC, $CDFILIAL){
		$params = array(	
			':CDTABEPREC'  => $CDTABEPREC,
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG
		);
		return $this->entityManager->getConnection()->fetchAll(self::GET_VIGENCIA_INTERVAL, $params);
	}
	
	public function findDataProcessamento($NRORG, $CDFILIAL){
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG
		);
		$RETORNO = [];
		
		$DATAPROCESSAMENTO = $this->entityManager->getConnection()->fetchAssoc(self::FIND_DATA_PROCESSAMENTO_PARAVEND, $params);
		
		$DATAPROCESSAMENTO = $DATAPROCESSAMENTO['DTPROCVND'];
        $PROXDATAPROCESSAMENTO = $this->finNextDate($NRORG, $CDFILIAL, $DATAPROCESSAMENTO);
        
        $RETORNO['DTPROCVND']     = str_replace('-', '/', $DATAPROCESSAMENTO);
        $RETORNO['PROXDTPROCVND'] 	= str_replace('-', '', $PROXDATAPROCESSAMENTO);
        return $RETORNO;
	}
	
	private function finNextDate($NRORG, $CDFILIAL, $DATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DATA' 	   => $DATA
		);
		
		$PROXDATE = date("d-m-Y", strtotime($DATA." + 1 day"));
        $params[':DATA'] = $PROXDATE;
        $DIAUTIL = $this->entityManager->getConnection()->fetchAssoc(self::PROX_DATA_PROCESSAMENTO, $params);
        if(!!$DIAUTIL){
        	$PROXDATE = $this->finNextDate($NRORG, $CDFILIAL, $PROXDATE);
        } 
        return $PROXDATE;
	}
	
	public function validate_PesqMapaECF_PesqCxEcf($NRORG, $CDFILIAL, $DATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DATA' 	   => $DATA
		);
		$PESQMAPAECF = $this->entityManager->getConnection()->fetchAssoc(self::VALIDATE_PESQMAPAECF, $params);
		$PESQCXECF = $this->entityManager->getConnection()->fetchAssoc(self::VALIDATE_PESQCXECF, $params);
		return (empty($PESQMAPAECF) && (!empty($PESQCXECF)));
	}
	
	public function validate_Dtmovturcaix($NRORG, $CDFILIAL, $DATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DTMOVTURCAIX' 	     => $DATA
		);
		$DTMOVTURCAIX = $this->entityManager->getConnection()->fetchAssoc(self::VALIDATE_DTMOVTURCAIX, $params);
		return !empty($DTMOVTURCAIX);
	}
	
	public function validate_notas($NRORG, $CDFILIAL, $DATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DATA'		   => $DATA
		);
		$NOTAS = $this->entityManager->getConnection()->fetchAssoc(self::VALIDATE_NOTAS, $params);
		return !empty($NOTAS);
	}
	
	public function validate_PesqContole($NRORG, $CDFILIAL, $DATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DATA'		   => $DATA
		);
		$PESQCONTOLE = $this->entityManager->getConnection()->fetchAll(self::VALIDATE_PESQCONTOLE, $params);
		$RETORNO = 0;
		foreach ($PESQCONTOLE as $value) {
			$params['CDCAIXA'] = $value['CDCAIXA'];
			$params['CDLOJA']  = $value['CDLOJA'];
			$PESQRETMALOTE = $this->entityManager->getConnection()->fetchAssoc(self::VALIDATE_PESQRETMALOTE, $params);
			if(!empty($PESQRETMALOTE)){
				$PESQVALORRET = $this->entityManager->getConnection()->fetchAssoc(self::ADQ_PESQVALORRET, $params);
				if(!empty($PESQVALORRET)){
					$SALDOANT = $this->entityManager->getConnection()->fetchAssoc(self::ADQ_SALDOANT, $params);
					if ($PESQVALORRET['VRMOVIVEND'] != $SALDOANT['VRSALDACUCXA'] ){
						$RETORNO = 2;
					}
				}else {
					$RETORNO = 3;
				}
			} 
		}
		return $RETORNO;
	}
	
	public function fechaDiaForSale($NRORG, $CDFILIAL, $DATA, $PROXDATA){
		$params = array(	
			':CDFILIAL'    => $CDFILIAL,
			':NRORG'       => $NRORG,
			':DATA'		   => $DATA,
			':PROXDATA'    => $PROXDATA
		);
		$adqPARAVEND = $this->entityManager->getConnection()->fetchAssoc(self::adqPARAVEND, $params);
		if($adqPARAVEND['IDCONSTCFECDIA'] == 'S'){}
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_DATAPROC, $params);
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_TURCAIXA, $params);
		return true;
	}
	
	const FIND_DATA_PROCESSAMENTO_PARAVEND = "
    SELECT TO_CHAR((NVL(DTPROCVND, SYSDATE)), 'DD-MM-YYYY') DTPROCVND
      FROM PARAVEND
     WHERE CDFILIAL = :CDFILIAL AND
           NRORG = :NRORG
	";

	const PROX_DATA_PROCESSAMENTO = "
	SELECT TO_CHAR((NVL(DTFERICARDVND, SYSDATE)), 'DD-MM-YYYY') DTFERICARDVND
        FROM FERICARDVND
    WHERE DTFERICARDVND = :DATA AND
       CDFILIAL      = :CDFILIAL AND
       IDTPFERICARDVND IN ('D','A')";
    
    const VALIDATE_PESQMAPAECF = "
    SELECT NRMAPARESECF
	  FROM MAPARESUMECF
	WHERE CDFILIAL     = :CDFILIAL AND
	      DTMAPARESECF = :DATA AND 
	      NRORG 	   = :NRORG
    ";
    
    const VALIDATE_PESQCXECF = "
    SELECT DISTINCT V.CDCAIXA
        FROM VENDA V , CAIXA C
       WHERE V.CDFILIAL    = :CDFILIAL
         AND V.DTENTRVENDA = :DATA
         AND V.NRORG = :NRORG
         AND V.NRORG = C.NRORG
         AND V.CDFILIAL = C.CDFILIAL
         AND V.CDCAIXA  = C.CDCAIXA
         AND C.IDHABCAIXAVENDA <>  'CAT'
         AND ((V.IDTPEMISVEND = 'E') OR (V.IDTPEMISVEND IS NULL))";
         
    const VALIDATE_DTMOVTURCAIX = "
    SELECT 1
        FROM TURCAIXA
       WHERE CDFILIAL = :CDFILIAL
         AND DTMOVTURCAIX = :DTMOVTURCAIX
         AND NRORG				= :NRORG
         AND DTFECHCAIX IS NULL";
         
    const VALIDATE_NOTAS = "
    SELECT *
	  FROM VENDA
	 WHERE CDFILIAL     = :CDFILIAL
	   AND NRORG		= :NRORG
	   AND DTENTRVENDA  = :DATA
	   AND IDSITUVENDA  = 'O'
	   AND IDTPEMISVEND = 'N'
	   AND IDTPAMBNFCE  = '1'
	   AND IDSTATUSNFCE <> 'A'";
	   
	const VALIDATE_PESQCONTOLE = "
	SELECT CDLOJA, CDCAIXA
		FROM CAIXA
	 WHERE CDFILIAL = :CDFILIAL AND
		   NRORG	= :NRORG AND
	       IDCONFVENDA = '2'";
	       
	const VALIDATE_PESQRETMALOTE = "
	SELECT CDLOJA, DTFERICARDVND, CDFILIAL
      FROM FERICARDVND
      WHERE CDFILIAL      = :CDFILIAL AND
            DTFERICARDVND = :DATA AND
            CDLOJA        = :CDLOJA AND
            NRORG         = :NRORG AND
            IDTPFERICARDVND IN ('A', 'M')";
            
	const ADQ_PESQVALORRET = "
	SELECT TRUNC(SUM(VRMOVIVEND),2) AS VRMOVIVEND, CDFILIAL
        FROM MOVCAIXA MC, TIPORECE TP
       WHERE
            MC.CDFILIAL     = :CDFILIAL   AND
            MC.DTMOVIMCAIXA = :DATA       AND
            MC.CDCAIXA      = :CDCAIXA    AND
            MC.NRORG        = :NRORG      AND
            MC.NRORG        = TP.NRORG    AND
            IDTIPOMOVIVE    = 'S'           AND
            MC.CDTIPORECE   = TP.CDTIPORECE AND
            TP.IDTIPORECE   = '4'
        GROUP BY CDFILIAL, DTMOVIMCAIXA, CDCAIXA
	";
	
	const ADQ_SALDOANT = "
	SELECT ROUND(VRSALDACUCXA,2) AS VRSALDACUCXA
      FROM SALDOCAIXA SD, TIPORECE TP
      WHERE SD.CDFILIAL = :CDFILIAL AND
            SD.CDCAIXA  = :CDCAIXA  AND
            SD.NRORG	= :NRORG	AND
            SD.NRORG	= TP.NRORG  AND
            SD.CDTIPORECE = TP.CDTIPORECE AND
            TP.IDTIPORECE = '4' AND
            DTMOVSALDCXA = (SELECT MAX(DTMOVSALDCXA)
                              FROM SALDOCAIXA
                              WHERE DTMOVSALDCXA < :DATA
                                AND CDFILIAL = :CDFILIAL
                                AND CDCAIXA  = :CDCAIXA )";
                                
	const adqPARAVEND = "
	SELECT
        NVL (IDGERACAOTC,'1') AS IDGERACAOTC,
        IDCONSTCFECDIA
      FROM PARAVEND
      WHERE CDFILIAL = :CDFILIAL AND 
    		NRORG	 = :NRORG
	";
	
	const UPDATE_DATAPROC = "
	UPDATE PARAVEND
         SET DTPROCVND = :PROXDATA
       WHERE CDFILIAL  = :CDFILIAL
    	 AND NRORG	   = :NRORG
	";
	
	const UPDATE_TURCAIXA = "
	UPDATE TURCAIXA
      SET IDCXLIBERADO = 'S'
      WHERE
      CDFILIAL = :CDFILIAL AND
      DTMOVTURCAIX = :DATA AND
      NRORG        = :NRORG
      ";
	}
	